/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


/* Includes */
#include "LIZARD.h"


/* Defines */
#define I2C_TIMEOUT_MAX  100


/* Declarations */
I2C_HandleTypeDef I2cHandle;
uint32_t i2c_timeout = I2C_TIMEOUT_MAX;


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Init
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Init( void )
{
   GPIO_InitTypeDef GPIO_InitStructure;

   if(HAL_I2C_GetState(&I2cHandle) == HAL_I2C_STATE_RESET)
   {
      __GPIOB_CLK_ENABLE();
      __GPIOE_CLK_ENABLE();

      I2cHandle.Instance              = I2C1;
      I2cHandle.Init.ClockSpeed       = 200000;
      I2cHandle.Init.DutyCycle        = I2C_DUTYCYCLE_2;
      I2cHandle.Init.OwnAddress1      = 0;
      I2cHandle.Init.AddressingMode   = I2C_ADDRESSINGMODE_7BIT;
      I2cHandle.Init.DualAddressMode  = I2C_DUALADDRESS_DISABLED;
      I2cHandle.Init.OwnAddress2      = 0;
      I2cHandle.Init.GeneralCallMode  = I2C_GENERALCALL_DISABLED;
      I2cHandle.Init.NoStretchMode    = I2C_NOSTRETCH_DISABLED;

      GPIO_InitStructure.Pin       = GPIO_PIN_6;
      GPIO_InitStructure.Mode      = GPIO_MODE_AF_OD;
      GPIO_InitStructure.Pull      = GPIO_PULLUP;
      GPIO_InitStructure.Speed     = GPIO_SPEED_FAST;
      GPIO_InitStructure.Alternate = GPIO_AF4_I2C1;
      HAL_GPIO_Init(GPIOB, &GPIO_InitStructure);
      GPIO_InitStructure.Pin = GPIO_PIN_7;
      HAL_GPIO_Init(GPIOB, &GPIO_InitStructure);

      __I2C1_CLK_ENABLE();
      __I2C1_FORCE_RESET();
      __I2C1_RELEASE_RESET();

      HAL_I2C_Init(&I2cHandle);
   }
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Error
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
static void LIZARD_I2C_Error( void )
{
   LED_ERROR_ON;
   HAL_Delay(20);
   HAL_I2C_DeInit(&I2cHandle);
   LIZARD_I2C_Init();
   LIZARD_TS_Reset();
   LED_ERROR_OFF;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Write_Data
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Write_Data(uint8_t Addr, uint8_t Reg, uint8_t Value)
{
   HAL_StatusTypeDef status = HAL_OK;
   status = HAL_I2C_Mem_Write(&I2cHandle, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT, &Value, 1, i2c_timeout);
   if(status != HAL_OK)
   {
      LIZARD_I2C_Error();
   }
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Write_Buffer
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Write_Buffer(uint8_t Addr, uint8_t Reg,  uint8_t *pBuffer, uint16_t Length)
{
   HAL_StatusTypeDef status = HAL_OK;
   status = HAL_I2C_Mem_Write(&I2cHandle, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT, pBuffer, Length, i2c_timeout);
   if(status != HAL_OK)
   {
      LIZARD_I2C_Error();
   }
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Read_Data
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_I2C_Read_Data(uint8_t Addr, uint8_t Reg)
{
   HAL_StatusTypeDef status = HAL_OK;
   uint8_t value = 0;
   status = HAL_I2C_Mem_Read(&I2cHandle, Addr, Reg, I2C_MEMADD_SIZE_8BIT, &value, 1, i2c_timeout);
   if(status != HAL_OK)
   {
	   LIZARD_I2C_Error();
   }
   return value;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Read_Buffer
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_I2C_Read_Buffer(uint8_t Addr, uint8_t Reg, uint8_t *pBuffer, uint16_t Length)
{
   HAL_StatusTypeDef status = HAL_OK;
   status = HAL_I2C_Mem_Read(&I2cHandle, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT, pBuffer, Length, i2c_timeout);
   if(status == HAL_OK)
   {
      return 0;
   }
   else
   {
	  LIZARD_I2C_Error();
      return 1;
   }
}
