/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


#ifndef __LIZARD_H
#define __LIZARD_H


/* Includes */
#include <stdio.h>
#include "stm32f4xx_hal.h"
#include "LIZARD_ts.h"

/* Defines */
/* Supported Display Types */
#define PH320240T023                    1   /* 3,5" TFT, 320 x 240 */
#define PH480272T009                    2   /* 4,3" TFT, 480 x 272 */
#define PH800480T024                    3   /* 5,0" TFT, 800 x 480 */
#define PH800480T013                    4   /* 7,0" TFT, 800 x 480 */
#define PH320240T023_PRO_X              5   /* 3,5" TFT, 320 x 240, PRO X Series */
#define PH102600T009                    6   /* 7,0" TFT, 1024 x 600 */
#define PH640480TXXX                    7   /* 5,7" TFT, 640 x 480 */

/* Display Configuration */
#ifndef LIZARD_DISPLAY_TYPE
  #define LIZARD_DISPLAY_TYPE PH480272T009
#endif

/* LED Configuration */
#define LED_BUSY_ON        				GPIOD->ODR |= GPIO_PIN_4
#define LED_BUSY_OFF       				GPIOD->ODR &= ~GPIO_PIN_4
#define LED_ERROR_ON       				GPIOD->ODR |= GPIO_PIN_3
#define LED_ERROR_OFF      				GPIOD->ODR &= ~GPIO_PIN_3

#if LIZARD_DISPLAY_TYPE == PH320240T023

   #define LIZARD_DISP_WIDTH          320
   #define LIZARD_DISP_HEIGHT         240
   #define LIZARD_DISP_TYPE           35
   #define LIZARD_DISP_THS            2
   #define LIZARD_DISP_THB            66
   #define LIZARD_DISP_THF            20
   #define LIZARD_DISP_TVS            2
   #define LIZARD_DISP_TVB            16
   #define LIZARD_DISP_TVF            4
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AH
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IIPC
   #define LIZARD_DISP_PLLSAIN        192
   #define LIZARD_DISP_PLLSAIR        4
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_4

#elif LIZARD_DISPLAY_TYPE == PH480272T009

   #define LIZARD_DISP_WIDTH          480
   #define LIZARD_DISP_HEIGHT         272
   #define LIZARD_DISP_TYPE           43
   #define LIZARD_DISP_THS            41
   #define LIZARD_DISP_THB            2
   #define LIZARD_DISP_THF            2
   #define LIZARD_DISP_TVS            10
   #define LIZARD_DISP_TVB            2
   #define LIZARD_DISP_TVF            2
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IPC
   #define LIZARD_DISP_PLLSAIN        320
   #define LIZARD_DISP_PLLSAIR        5
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_4

#elif LIZARD_DISPLAY_TYPE == PH800480T024

   #define LIZARD_DISP_WIDTH          800
   #define LIZARD_DISP_HEIGHT         480
   #define LIZARD_DISP_TYPE           50
   #define LIZARD_DISP_THS            128
   #define LIZARD_DISP_THB            88
   #define LIZARD_DISP_THF            40
   #define LIZARD_DISP_TVS            2
   #define LIZARD_DISP_TVB            33
   #define LIZARD_DISP_TVF            10
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IPC
   #define LIZARD_DISP_PLLSAIN        320
   #define LIZARD_DISP_PLLSAIR        5
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_2

#elif LIZARD_DISPLAY_TYPE == PH800480T013

   #define LIZARD_DISP_WIDTH          800
   #define LIZARD_DISP_HEIGHT         480
   #define LIZARD_DISP_TYPE           70
   #define LIZARD_DISP_THS            128
   #define LIZARD_DISP_THB            88
   #define LIZARD_DISP_THF            40
   #define LIZARD_DISP_TVS            2
   #define LIZARD_DISP_TVB            33
   #define LIZARD_DISP_TVF            10
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IPC
   #define LIZARD_DISP_PLLSAIN        192
   #define LIZARD_DISP_PLLSAIR        4
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_2

#elif LIZARD_DISPLAY_TYPE == PH320240T023_PRO_X

   #define LIZARD_DISP_WIDTH          320
   #define LIZARD_DISP_HEIGHT         240
   #define LIZARD_DISP_TYPE           35
   #define LIZARD_DISP_THS            2
   #define LIZARD_DISP_THB            66
   #define LIZARD_DISP_THF            20
   #define LIZARD_DISP_TVS            2
   #define LIZARD_DISP_TVB            16
   #define LIZARD_DISP_TVF            4
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IIPC
   #define LIZARD_DISP_PLLSAIN        192
   #define LIZARD_DISP_PLLSAIR        4
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_4

#elif LIZARD_DISPLAY_TYPE == PH102600T009

   #define LIZARD_DISP_WIDTH          1024
   #define LIZARD_DISP_HEIGHT         600
   #define LIZARD_DISP_TYPE           70
   #define LIZARD_DISP_THS            10
   #define LIZARD_DISP_THB            150
   #define LIZARD_DISP_THF            20
   #define LIZARD_DISP_TVS            10
   #define LIZARD_DISP_TVB            13
   #define LIZARD_DISP_TVF            12
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IPC
   #define LIZARD_DISP_PLLSAIN        192
   #define LIZARD_DISP_PLLSAIR        2
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_2

#elif LIZARD_DISPLAY_TYPE == PH640480TXXX

   #define LIZARD_DISP_WIDTH          640
   #define LIZARD_DISP_HEIGHT         480
   #define LIZARD_DISP_TYPE           50
   #define LIZARD_DISP_THS            30
   #define LIZARD_DISP_THB            114
   #define LIZARD_DISP_THF            16
   #define LIZARD_DISP_TVS            3
   #define LIZARD_DISP_TVB            32
   #define LIZARD_DISP_TVF            10
   #define LIZARD_DISP_HSPOL          LTDC_HSPOLARITY_AL
   #define LIZARD_DISP_VSPOL          LTDC_VSPOLARITY_AL
   #define LIZARD_DISP_DEPOL          LTDC_DEPOLARITY_AL
   #define LIZARD_DISP_PCLKPOL        LTDC_PCPOLARITY_IPC
   #define LIZARD_DISP_PLLSAIN        192
   #define LIZARD_DISP_PLLSAIR        4
   #define LIZARD_DISP_PLLSAIDivR     RCC_PLLSAIDIVR_2

#else

  #error unknown LIZARD_DISPLAY_TYPE

#endif


/* Declarations */


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Init
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Init
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Write_Data
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Write_Data
(
  uint8_t Addr,
  uint8_t Reg,
  uint8_t Value
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Write_Buffer
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_I2C_Write_Buffer
(
  uint8_t Addr,
  uint8_t Reg,
  uint8_t *pBuffer,
  uint16_t Length
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Read_Data
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_I2C_Read_Data
(
  uint8_t Addr,
  uint8_t Reg
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_I2C_Read_Buffer
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_I2C_Read_Buffer
(
  uint8_t Addr,
  uint8_t Reg,
  uint8_t *pBuffer,
  uint16_t Length
);


#endif	// __LIZARD_H


