/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


#ifndef __LIZARD_QSPI_H
#define __LIZARD_QSPI_H


/* Includes */
#include "LIZARD.h"
#include "mt25ql512.h"
#include "s25fl512s.h"


/* Defines */
#define QSPI_S25FL512S     				((uint8_t)0x20)
#define QSPI_N25Q512A      				((uint8_t)0x40)

#define JEDEC_MANUF_ID_MICRON   		((uint8_t) 0x20)
#define JEDEC_MANUF_ID_SPANSION 		((uint8_t) 0x01)

#define QSPI_OK            				((uint8_t)0x00)
#define QSPI_ERROR         				((uint8_t)0x01)
#define QSPI_BUSY          				((uint8_t)0x02)
#define QSPI_NOT_SUPPORTED 				((uint8_t)0x04)
#define QSPI_SUSPENDED     				((uint8_t)0x08)
#define QSPI_PROTECTED     				((uint8_t)0x10)

#define QSPI_CLK_ENABLE()             	__HAL_RCC_QSPI_CLK_ENABLE()
#define QSPI_CLK_DISABLE()            	__HAL_RCC_QSPI_CLK_DISABLE()
#define QSPI_CS_GPIO_CLK_ENABLE()     	__HAL_RCC_GPIOB_CLK_ENABLE()
#define QSPI_CS_GPIO_CLK_DISABLE()  	__HAL_RCC_GPIOB_CLK_DISABLE()
#define QSPI_DX_CLK_GPIO_CLK_ENABLE() 	__HAL_RCC_GPIOF_CLK_ENABLE()
#define QSPI_DX_CLK_GPIO_CLK_DISABLE()  __HAL_RCC_GPIOF_CLK_DISABLE()

#define QSPI_FORCE_RESET()            	__HAL_RCC_QSPI_FORCE_RESET()
#define QSPI_RELEASE_RESET()          	__HAL_RCC_QSPI_RELEASE_RESET()


/* Declarations */
typedef struct
{
  uint32_t FlashSize;          /*!< Size of the flash                         */
  uint32_t EraseSectorSize;    /*!< Size of sectors for the erase operation   */
  uint32_t EraseSectorsNumber; /*!< Number of sectors for the erase operation */
  uint32_t ProgPageSize;       /*!< Size of pages for the program operation   */
  uint32_t ProgPagesNumber;    /*!< Number of pages for the program operation */
  uint32_t DummyCyclesRead;    /*!< Number of dummy read cycles               */
  uint8_t  ManufID;            /*!< Manufacturor value                        */
  uint32_t SectorEraseMaxTime; /*!< Sector Erase max value                    */
  uint32_t BulkEraseMaxTime;   /*!< Bulk Erase Max Time                       */
} QSPI_InfoTypeDef;


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_Init
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_Init
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_DeInit
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_DeInit
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_Read
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_Read
(
  uint8_t* pData,
  uint32_t ReadAddr,
  uint32_t Size
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_Write
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_Write
(
  uint8_t* pData,
  uint32_t WriteAddr,
  uint32_t Size
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_Erase_Block
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_Erase_Block
(
  uint32_t BlockAddress
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_Erase_Chip
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_Erase_Chip
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_GetStatus
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_GetStatus
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_GetInfo
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_GetInfo
(
  QSPI_InfoTypeDef* pInfo
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_QSPI_EnableMemoryMappedMode
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_QSPI_EnableMemoryMappedMode
(
  void
);


/*******************************************************************************
* FUNCTION:
*   BSP_QSPI_MspInit
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void BSP_QSPI_MspInit
(
  QSPI_HandleTypeDef *hqspi,
  void *Params
);


#endif 	// __LIZARD_QSPI_H
