/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


/* Includes */
#include "LIZARD_sdram.h"


/* Defines */


/* Declarations */
static SDRAM_HandleTypeDef SdramHandle;
static FMC_SDRAM_TimingTypeDef Timing;
static FMC_SDRAM_CommandTypeDef Command;


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_DMA_IRQHandler
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: ISR fr DMA
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_DMA_IRQHandler(void)
{
   HAL_DMA_IRQHandler(SdramHandle.hdma); 
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Init
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: SD-RAM initialisieren
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Init( void )
{
   static DMA_HandleTypeDef dmaHandle;
   GPIO_InitTypeDef GPIO_InitStructure;
   SDRAM_HandleTypeDef  *hsdram = &SdramHandle;

   /* SDRAM device configuration */
   SdramHandle.Instance = FMC_SDRAM_DEVICE;

   /* FMC Configuration -------------------------------------------------------*/
   /* FMC SDRAM Bank configuration */
   /* Timing configuration for 90 Mhz of SD clock frequency (180Mhz/2) */
   /* TMRD: 2 Clock cycles */
   Timing.LoadToActiveDelay    = 2;
   /* TXSR: min=70ns (7x11.11ns) */
   Timing.ExitSelfRefreshDelay = 7;
   /* TRAS: min=42ns (4x11.11ns) max=120k (ns) */
   Timing.SelfRefreshTime      = 4;
   /* TRC:  min=70 (7x11.11ns) */
   Timing.RowCycleDelay        = 7;
   /* TWR:  min=1+ 7ns (1+1x11.11ns) */
   Timing.WriteRecoveryTime    = 2;
   /* TRP:  20ns => 2x11.11ns*/
   Timing.RPDelay              = 2;
   /* TRCD: 20ns => 2x11.11ns */
   Timing.RCDDelay             = 2;
   
   /* FMC SDRAM control configuration */
   SdramHandle.Init.SDBank             = FMC_SDRAM_BANK1;
   /* Row addressing: [7:0] */
   SdramHandle.Init.ColumnBitsNumber   = FMC_SDRAM_COLUMN_BITS_NUM_8;
   /* Column addressing: [11:0] */
   SdramHandle.Init.RowBitsNumber      = FMC_SDRAM_ROW_BITS_NUM_11;
   SdramHandle.Init.MemoryDataWidth    = SDRAM_MEMORY_WIDTH;
   SdramHandle.Init.InternalBankNumber = FMC_SDRAM_INTERN_BANKS_NUM_4;
   SdramHandle.Init.CASLatency         = SDRAM_CAS_LATENCY;
   SdramHandle.Init.WriteProtection    = FMC_SDRAM_WRITE_PROTECTION_DISABLE;
   SdramHandle.Init.SDClockPeriod      = SDCLOCK_PERIOD;
   SdramHandle.Init.ReadBurst          = SDRAM_READBURST;
   SdramHandle.Init.ReadPipeDelay      = FMC_SDRAM_RPIPE_DELAY_0;
                     
   /* SDRAM controller initialization */
   /* Enable FMC clock */
   __FMC_CLK_ENABLE();

   /* Enable chosen DMAx clock */
   __DMAx_CLK_ENABLE();

   /* Enable GPIOs clock */
   __GPIOD_CLK_ENABLE();
   __GPIOE_CLK_ENABLE();
   __GPIOF_CLK_ENABLE();
   __GPIOG_CLK_ENABLE();
   __GPIOH_CLK_ENABLE();
   __GPIOI_CLK_ENABLE();
                             
   /* Common GPIO configuration */
   GPIO_InitStructure.Mode  = GPIO_MODE_AF_PP;
   GPIO_InitStructure.Speed = GPIO_SPEED_FAST;
   GPIO_InitStructure.Pull  = GPIO_NOPULL;
   GPIO_InitStructure.Alternate = GPIO_AF12_FMC;
 
   /* GPIOD configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_0 | GPIO_PIN_1  | GPIO_PIN_8 |
                            GPIO_PIN_9 | GPIO_PIN_10 | GPIO_PIN_14 |
                            GPIO_PIN_15;
   HAL_GPIO_Init(GPIOD, &GPIO_InitStructure);

   /* GPIOE configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_0  | GPIO_PIN_1  | GPIO_PIN_7 |
                            GPIO_PIN_8  | GPIO_PIN_9  | GPIO_PIN_10 |
                            GPIO_PIN_11 | GPIO_PIN_12 | GPIO_PIN_13 |
                            GPIO_PIN_14 | GPIO_PIN_15;
   HAL_GPIO_Init(GPIOE, &GPIO_InitStructure);

   /* GPIOF configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_0  | GPIO_PIN_1 | GPIO_PIN_2 | 
                            GPIO_PIN_3  | GPIO_PIN_4 | GPIO_PIN_5 |
                            GPIO_PIN_11 | GPIO_PIN_12 | GPIO_PIN_13 |
                            GPIO_PIN_14 | GPIO_PIN_15;
   HAL_GPIO_Init(GPIOF, &GPIO_InitStructure);

   /* GPIOG configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_4 |
                            GPIO_PIN_5 | GPIO_PIN_8 | GPIO_PIN_15;
   HAL_GPIO_Init(GPIOG, &GPIO_InitStructure);


   /* GPIOH configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_5 |
                            GPIO_PIN_8 | GPIO_PIN_9 | GPIO_PIN_10|
                            GPIO_PIN_11 | GPIO_PIN_12 | GPIO_PIN_13|
                            GPIO_PIN_14 | GPIO_PIN_15;
   HAL_GPIO_Init(GPIOH, &GPIO_InitStructure);

   /* GPIOI configuration */
   GPIO_InitStructure.Pin = GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 |
                            GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5 |
                            GPIO_PIN_6 | GPIO_PIN_7 | GPIO_PIN_8|
                            GPIO_PIN_9 | GPIO_PIN_10;
   HAL_GPIO_Init(GPIOI, &GPIO_InitStructure);


   /* Configure common DMA parameters */
   dmaHandle.Init.Channel             = SDRAM_DMAx_CHANNEL;
   dmaHandle.Init.Direction           = DMA_MEMORY_TO_MEMORY;
   dmaHandle.Init.PeriphInc           = DMA_PINC_ENABLE;
   dmaHandle.Init.MemInc              = DMA_MINC_ENABLE;
   dmaHandle.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
   dmaHandle.Init.MemDataAlignment    = DMA_MDATAALIGN_WORD;
   dmaHandle.Init.Mode                = DMA_NORMAL;
   dmaHandle.Init.Priority            = DMA_PRIORITY_HIGH;
   dmaHandle.Init.FIFOMode            = DMA_FIFOMODE_DISABLE;
   dmaHandle.Init.FIFOThreshold       = DMA_FIFO_THRESHOLD_FULL;
   dmaHandle.Init.MemBurst            = DMA_MBURST_SINGLE;
   dmaHandle.Init.PeriphBurst         = DMA_PBURST_SINGLE; 
  
   dmaHandle.Instance = SDRAM_DMAx_STREAM;
  
   /* Associate the DMA handle */
   __HAL_LINKDMA(hsdram, hdma, dmaHandle);
  
   /* Deinitialize the stream for new transfer */
   HAL_DMA_DeInit(&dmaHandle);
  
   /* Configure the DMA stream */
   HAL_DMA_Init(&dmaHandle); 
  
   /* NVIC configuration for DMA transfer complete interrupt */
   HAL_NVIC_SetPriority(SDRAM_DMAx_IRQn, 0, 0);
   HAL_NVIC_EnableIRQ(SDRAM_DMAx_IRQn);
   HAL_SDRAM_Init(&SdramHandle, &Timing);
  
   /* SDRAM initialization sequence */
   __IO uint32_t tmpmrd =0;
  
   /* Step 1:  Configure a clock configuration enable command */
   Command.CommandMode             = FMC_SDRAM_CMD_CLK_ENABLE;
   Command.CommandTarget           = FMC_SDRAM_CMD_TARGET_BANK1;
   Command.AutoRefreshNumber       = 1;
   Command.ModeRegisterDefinition  = 0;

   /* Send the command */
   HAL_SDRAM_SendCommand(&SdramHandle, &Command, SDRAM_TIMEOUT);

   /* Step 2: Insert 100 us minimum delay */ 
   /* Inserted delay is equal to 1 ms due to systick time base unit (ms) */
   HAL_Delay(1);

   /* Step 3: Configure a PALL (precharge all) command */ 
   Command.CommandMode             = FMC_SDRAM_CMD_PALL;
   Command.CommandTarget           = FMC_SDRAM_CMD_TARGET_BANK1;
   Command.AutoRefreshNumber       = 1;
   Command.ModeRegisterDefinition  = 0;

   /* Send the command */
   HAL_SDRAM_SendCommand(&SdramHandle, &Command, SDRAM_TIMEOUT);  
  
   /* Step 4: Configure an Auto Refresh command */ 
   Command.CommandMode             = FMC_SDRAM_CMD_AUTOREFRESH_MODE;
   Command.CommandTarget           = FMC_SDRAM_CMD_TARGET_BANK1;
   Command.AutoRefreshNumber       = 8;
   Command.ModeRegisterDefinition  = 0;

   /* Send the command */
   HAL_SDRAM_SendCommand(&SdramHandle, &Command, SDRAM_TIMEOUT);
  
   /* Step 5: Program the external memory mode register */
   tmpmrd = (uint32_t)SDRAM_MODEREG_BURST_LENGTH_1          |
                      SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL   |
                      SDRAM_MODEREG_CAS_LATENCY_3           |
                      SDRAM_MODEREG_OPERATING_MODE_STANDARD |
                      SDRAM_MODEREG_WRITEBURST_MODE_SINGLE;
  
   Command.CommandMode             = FMC_SDRAM_CMD_LOAD_MODE;
   Command.CommandTarget           = FMC_SDRAM_CMD_TARGET_BANK1;
   Command.AutoRefreshNumber       = 1;
   Command.ModeRegisterDefinition  = tmpmrd;

   /* Send the command */
   HAL_SDRAM_SendCommand(&SdramHandle, &Command, SDRAM_TIMEOUT);
  
   /* Step 6: Set the refresh rate counter */
   /* Set the device refresh rate */
   HAL_SDRAM_ProgramRefreshRate(&SdramHandle, REFRESH_COUNT); 
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_8
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 8 Bit in das SD-RAM schreiben
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_8(uint32_t adr, uint8_t data)
{
   *(uint8_t*) (SDRAM_DEVICE_ADDR + adr) = data;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_16
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 16 Bit in das SD-RAM schreiben
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_16(uint32_t adr, uint16_t data)
{
   *(uint16_t*) (SDRAM_DEVICE_ADDR + adr) = data;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_32
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 32 Bit in das SD-RAM schreiben
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_32(uint32_t adr, uint32_t data)
{
   *(uint32_t*) (SDRAM_DEVICE_ADDR + adr) = data;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_8
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 8 Bit aus dem SD-RAM lesen
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_SDRAM_Read_8(uint32_t adr)
{
   return *(volatile uint8_t*)(SDRAM_DEVICE_ADDR + adr);
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_16
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 16 Bit aus dem SD-RAM lesen
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint16_t LIZARD_SDRAM_Read_16(uint32_t adr)
{
   return *(volatile uint16_t*)(SDRAM_DEVICE_ADDR + adr);
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_32
*
* DESCRIPTION:
*   __DESCRIPTION__ --> OLD: 32 Bit aus dem SD-RAM lesen
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint32_t LIZARD_SDRAM_Read_32(uint32_t adr)
{
   return *(volatile uint32_t*)(SDRAM_DEVICE_ADDR + adr);
}


