/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


#ifndef __LIZARD_SDRAM_H
#define __LIZARD_SDRAM_H


/* Includes */
#include "LIZARD.h"


/* Defines */
#define SDRAM_DEVICE_ADDR                        ((uint32_t)0xC0000000)
#define SDRAM_DEVICE_SIZE                        ((uint32_t)0x800000)       	/* SDRAM device size in MBytes */
#define SDRAM_OK                                 0
#define SDRAM_FAIL                               -1

#define SDRAM_MEMORY_WIDTH                       FMC_SDRAM_MEM_BUS_WIDTH_32
#define SDRAM_CAS_LATENCY                        FMC_SDRAM_CAS_LATENCY_3 
#define SDCLOCK_PERIOD                           FMC_SDRAM_CLOCK_PERIOD_2    	/* Default configuration used with LCD */
#define SDRAM_READBURST                          FMC_SDRAM_RBURST_DISABLE    	/* Default configuration used with LCD */

#define REFRESH_COUNT                            ((uint32_t)1386)            	/* SDRAM refresh counter */
#define SDRAM_TIMEOUT                            ((uint32_t)0xFFFF)

#define __DMAx_CLK_ENABLE                        __DMA2_CLK_ENABLE

#define SDRAM_DMAx_CHANNEL                       DMA_CHANNEL_0

#define SDRAM_DMAx_STREAM                        DMA2_Stream0
#define SDRAM_DMAx_IRQn                          DMA2_Stream0_IRQn
#define SDRAM_DMAx_IRQHandler                    DMA2_Stream0_IRQHandler

#define SDRAM_MODEREG_BURST_LENGTH_1             ((uint16_t)0x0000)
#define SDRAM_MODEREG_BURST_LENGTH_2             ((uint16_t)0x0001)
#define SDRAM_MODEREG_BURST_LENGTH_4             ((uint16_t)0x0002)
#define SDRAM_MODEREG_BURST_LENGTH_8             ((uint16_t)0x0004)
#define SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL      ((uint16_t)0x0000)
#define SDRAM_MODEREG_BURST_TYPE_INTERLEAVED     ((uint16_t)0x0008)
#define SDRAM_MODEREG_CAS_LATENCY_2              ((uint16_t)0x0020)
#define SDRAM_MODEREG_CAS_LATENCY_3              ((uint16_t)0x0030)
#define SDRAM_MODEREG_OPERATING_MODE_STANDARD    ((uint16_t)0x0000)
#define SDRAM_MODEREG_WRITEBURST_MODE_PROGRAMMED ((uint16_t)0x0000)
#define SDRAM_MODEREG_WRITEBURST_MODE_SINGLE     ((uint16_t)0x0200)


/* Declarations */


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Init
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Init
(
  void
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_8
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_8
(
  uint32_t adr,
  uint8_t data
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_16
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_16
(
  uint32_t adr,
  uint16_t data
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Write_32
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
void LIZARD_SDRAM_Write_32
(
  uint32_t adr,
  uint32_t data
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_8
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint8_t LIZARD_SDRAM_Read_8
(
  uint32_t adr
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_16
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint16_t LIZARD_SDRAM_Read_16
(
  uint32_t adr
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Read_32
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
uint32_t LIZARD_SDRAM_Read_32
(
  uint32_t adr
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_SDRAM_Test
*
* DESCRIPTION:
*   __DESCRIPTION__
*
* ARGUMENTS:
*   __ARGUMENTS__
*
* RETURN VALUE:
*   __RETURN_VALUE__
*
*******************************************************************************/
int LIZARD_SDRAM_Test
(
  void
);


#endif	// __LIZARD_SDRAM_H
