/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*                                               and Maximilian Schwarz (DeeFuse)
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   The module LIZARD_ts is responsible to read the touch states from the
*   touch hardware via I2C. The touch driver supports multi-touch.
*
*******************************************************************************/

#ifndef __LIZARD_TS_H
#define __LIZARD_TS_H

#include "LIZARD.h"

#define NO_OF_FINGERS 				10

typedef struct
{
  uint8_t  touchDetected;              /*!< Total number of active touches detected at last scan */
  uint16_t touchX[NO_OF_FINGERS];      /*!< Touch X[0], X[1] coordinates on 12 bits */
  uint16_t touchY[NO_OF_FINGERS];      /*!< Touch Y[0], Y[1] coordinates on 12 bits */
  uint8_t  touchId[NO_OF_FINGERS];
} TS_StateTypeDef;


/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_Init
*
* DESCRIPTION:
*   Initialize the touch driver hardware.
*
* ARGUMENTS:
*   screen width in pixels
*   screen height in pixels
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void LIZARD_TS_Init
(
  int screenWidth,
  int screenHeight
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_GetState
*
* DESCRIPTION:
*   Read the current touch coordinates and touch ids from the touch data buffer
*   and return the current touch information in the provided data strucutre.
*
* ARGUMENTS:
*   Pointer to structure containing last touch information. In case of new
*   touch events, the structure will return current touch information.
*
* RETURN VALUE:
*   Returns the number of detected touch events, otherwise 0.
*
*******************************************************************************/
uint8_t LIZARD_TS_Get_State
(
  TS_StateTypeDef* TS_State
);


/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_Reset
*
* DESCRIPTION:
*   Reset the touch driver.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void LIZARD_TS_Reset
(
  void
);

#endif	// __LIZARD_TS_H
