/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   __DESCRIPTION__
*
*******************************************************************************/


#ifndef __MT25QL512
#define __MT25QL512


/* Includes */


/* Defines */
/* N25Q512A Micron Memory Configuration */
#define MT25QL512_FLASH_SIZE                  			0x4000000 			/* 512 MBits => 64MBytes */
#define MT25QL512_SECTOR_SIZE                 			0x10000   			/* 1024 sectors of 64KBytes */
#define MT25QL512_SUBSECTOR_SIZE              			0x1000    			/* 16384 subsectors of 4kBytes */
#define MT25QL512_PAGE_SIZE                   			0x100     			/* 262144 pages of 256 bytes */

#define MT25QL512_DUMMY_CYCLES_READ           			8
#define MT25QL512_DUMMY_CYCLES_READ_QUAD      			10

#define MT25QL512_DUMMY_CYCLES_READ_DTR       			6
#define MT25QL512_DUMMY_CYCLES_READ_QUAD_DTR  			8

#define MT25QL512_BULK_ERASE_MAX_TIME         			480000
#define MT25QL512_SECTOR_ERASE_MAX_TIME       			3000
#define MT25QL512_SUBSECTOR_ERASE_MAX_TIME    			800

/* Reset Operations */
#define MT25QL512_RESET_ENABLE_CMD                     	0x66
#define MT25QL512_RESET_MEMORY_CMD                     	0x99

/* Identification Operations */
#define MT25QL512_READ_ID_CMD                          	0x9E
#define MT25QL512_READ_ID_CMD2                         	0x9F
#define MT25QL512_MULTIPLE_IO_READ_ID_CMD              	0xAF
#define MT25QL512_READ_SERIAL_FLASH_DISCO_PARAM_CMD    	0x5A

/* Read Operations */
#define MT25QL512_READ_CMD                             	0x03
#define MT25QL512_READ_4_BYTE_ADDR_CMD                 	0x13
#define MT25QL512_FAST_READ_CMD                        	0x0B
#define MT25QL512_FAST_READ_DTR_CMD                    	0x0D
#define MT25QL512_FAST_READ_4_BYTE_ADDR_CMD            	0x0C
#define MT25QL512_DUAL_OUT_FAST_READ_CMD               	0x3B
#define MT25QL512_DUAL_OUT_FAST_READ_DTR_CMD           	0x3D
#define MT25QL512_DUAL_OUT_FAST_READ_4_BYTE_ADDR_CMD   	0x3C
#define MT25QL512_DUAL_INOUT_FAST_READ_CMD             	0xBB
#define MT25QL512_DUAL_INOUT_FAST_READ_DTR_CMD         	0xBD
#define MT25QL512_DUAL_INOUT_FAST_READ_4_BYTE_ADDR_CMD 	0xBC
#define MT25QL512_QUAD_OUT_FAST_READ_CMD               	0x6B
#define MT25QL512_QUAD_OUT_FAST_READ_DTR_CMD           	0x6D
#define MT25QL512_QUAD_OUT_FAST_READ_4_BYTE_ADDR_CMD   	0x6C
#define MT25QL512_QUAD_INOUT_FAST_READ_CMD             	0xEB
#define MT25QL512_QUAD_INOUT_FAST_READ_DTR_CMD         	0xED
#define MT25QL512_QUAD_INOUT_FAST_READ_4_BYTE_ADDR_CMD 	0xEC

/* Write Operations */
#define MT25QL512_WRITE_ENABLE_CMD                     	0x06
#define MT25QL512_WRITE_DISABLE_CMD                    	0x04

/* Register Operations */
#define MT25QL512_READ_STATUS_REG_CMD                  	0x05
#define MT25QL512_WRITE_STATUS_REG_CMD                 	0x01

#define MT25QL512_READ_LOCK_REG_CMD                    	0xE8
#define MT25QL512_WRITE_LOCK_REG_CMD                   	0xE5

#define MT25QL512_READ_FLAG_STATUS_REG_CMD             	0x70
#define MT25QL512_CLEAR_FLAG_STATUS_REG_CMD            	0x50

#define MT25QL512_READ_NONVOL_CFG_REG_CMD              	0xB5
#define MT25QL512_WRITE_NONVOL_CFG_REG_CMD             	0xB1

#define MT25QL512_READ_VOL_CFG_REG_CMD                 	0x85
#define MT25QL512_WRITE_VOL_CFG_REG_CMD                	0x81

#define MT25QL512_READ_ENHANCED_VOL_CFG_REG_CMD        	0x65
#define MT25QL512_WRITE_ENHANCED_VOL_CFG_REG_CMD       	0x61

#define MT25QL512_READ_EXT_ADDR_REG_CMD                	0xC8
#define MT25QL512_WRITE_EXT_ADDR_REG_CMD               	0xC5

/* Program Operations */
#define MT25QL512_PAGE_PROG_CMD                        	0x02
#define MT25QL512_PAGE_PROG_4_BYTE_ADDR_CMD            	0x12
#define MT25QL512_DUAL_IN_FAST_PROG_CMD                	0xA2
#define MT25QL512_EXT_DUAL_IN_FAST_PROG_CMD            	0xD2
#define MT25QL512_QUAD_IN_FAST_PROG_CMD                	0x32
#define MT25QL512_EXT_QUAD_IN_FAST_PROG_CMD            	0x12
#define MT25QL512_QUAD_IN_FAST_PROG_4_BYTE_ADDR_CMD    	0x34

/* Erase Operations */
#define MT25QL512_SUBSECTOR_ERASE_CMD                  	0x20
#define MT25QL512_SUBSECTOR_ERASE_4_BYTE_ADDR_CMD      	0x21
#define MT25QL512_SECTOR_ERASE_CMD                     	0xD8
#define MT25QL512_SECTOR_ERASE_4_BYTE_ADDR_CMD         	0xDC
#define MT25QL512_BULK_ERASE_CMD                       	0xC7

#define MT25QL512_PROG_ERASE_RESUME_CMD                	0x7A
#define MT25QL512_PROG_ERASE_SUSPEND_CMD               	0x75

/* One-Time Programmable Operations */
#define MT25QL512_READ_OTP_ARRAY_CMD                   	0x4B
#define MT25QL512_PROG_OTP_ARRAY_CMD                   	0x42

/* Address Mode Operations */
#define MT25QL512_ENTER_4_BYTE_ADDR_MODE_CMD           	0xB7
#define MT25QL512_EXIT_4_BYTE_ADDR_MODE_CMD            	0xE9
#define MT25QL512_ENTER_QUAD_CMD                       	0x35
#define MT25QL512_EXIT_QUAD_CMD                        	0xF5

/* Status Register */
#define MT25QL512_SR_WIP                      			((uint8_t)0x01)    		/*!< Write in progress */
#define MT25QL512_SR_WREN                     			((uint8_t)0x02)    		/*!< Write enable latch */
#define MT25QL512_SR_BLOCKPR                  			((uint8_t)0x5C)    		/*!< Block protected against program and erase operations */
#define MT25QL512_SR_PRBOTTOM                 			((uint8_t)0x20)    		/*!< Protected memory area defined by BLOCKPR starts from top or bottom */
#define MT25QL512_SR_SRWREN                   			((uint8_t)0x80)    		/*!< Status register write enable/disable */

/* Non volatile Configuration Register */
#define MT25QL512_NVCR_NBADDR                 			((uint16_t)0x0001) 		/*!< 3-bytes or 4-bytes addressing */
#define MT25QL512_NVCR_SEGMENT                			((uint16_t)0x0002) 		/*!< Upper or lower 128Mb segment selected by default */
#define MT25QL512_NVCR_DUAL                   			((uint16_t)0x0004) 		/*!< Dual I/O protocol */
#define MT25QL512_NVCR_QUAB                   			((uint16_t)0x0008) 		/*!< Quad I/O protocol */
#define MT25QL512_NVCR_RH                     			((uint16_t)0x0010) 		/*!< Reset/hold */
#define MT25QL512_NVCR_ODS                    			((uint16_t)0x01C0) 		/*!< Output driver strength */
#define MT25QL512_NVCR_XIP                    			((uint16_t)0x0E00) 		/*!< XIP mode at power-on reset */
#define MT25QL512_NVCR_NB_DUMMY               			((uint16_t)0xF000) 		/*!< Number of dummy clock cycles */

/* Volatile Configuration Register */
#define MT25QL512_VCR_WRAP                    			((uint8_t)0x03)    		/*!< Wrap */
#define MT25QL512_VCR_XIP                     			((uint8_t)0x08)    		/*!< XIP */
#define MT25QL512_VCR_NB_DUMMY                			((uint8_t)0xF0)    		/*!< Number of dummy clock cycles */

/* Extended Address Register */
#define MT25QL512_EAR_A24                     			((uint8_t)0x01)    		/*!< Select the lower or upper 128Mb segment */

/* Enhanced Volatile Configuration Register */
#define MT25QL512_EVCR_ODS                    			((uint8_t)0x07)    		/*!< Output driver strength */
#define MT25QL512_EVCR_VPPA                   			((uint8_t)0x08)    		/*!< Vpp accelerator */
#define MT25QL512_EVCR_RH                     			((uint8_t)0x10)    		/*!< Reset/hold */
#define MT25QL512_EVCR_DUAL                   			((uint8_t)0x40)    		/*!< Dual I/O protocol */
#define MT25QL512_EVCR_QUAD                   			((uint8_t)0x80)    		/*!< Quad I/O protocol */

/* Flag Status Register */
#define N25Q512Az_FSR_NBADDR                  			((uint8_t)0x01)    		/*!< 3-bytes or 4-bytes addressing */
#define MT25QL512_FSR_PRERR                   			((uint8_t)0x02)    		/*!< Protection error */
#define MT25QL512_FSR_PGSUS                   			((uint8_t)0x04)    		/*!< Program operation suspended */
#define MT25QL512_FSR_VPPERR                  			((uint8_t)0x08)    		/*!< Invalid voltage during program or erase */
#define MT25QL512_FSR_PGERR                   			((uint8_t)0x10)    		/*!< Program error */
#define MT25QL512_FSR_ERERR                   			((uint8_t)0x20)    		/*!< Erase error */
#define MT25QL512_FSR_ERSUS                   			((uint8_t)0x40)    		/*!< Erase operation suspended */
#define MT25QL512_FSR_READY                   			((uint8_t)0x80)    		/*!< Ready or command in progress */


/* Declarations */


#endif

